//
//
//

#include "dgrid.h"

DGrid::DGrid(QWidget *parent):QTableView(parent)
{
	int temp;
	setTableFlags(Tbl_autoScrollBars | Tbl_clipCellPainting);
	for(temp=0;temp<MAX_DGRID_COLS;temp++)
	{
		setWidth(temp,50);
	};
	curCol=0;
	curRow=0;
	//setNumRows(100);
	//setNumCols(2);
	//setCellWidth(65);
	setCellHeight(20);
	//setCursor(ibeamCursor);
	setFrameStyle(Panel | Sunken);
	setFocusPolicy(StrongFocus);
	text=NULL;
	//font.setCharSet(Latin5);
	gridfont=new QFont("courier",14,QFont::Normal,FALSE,QFont::Latin5);
	//font.setPointSize(14);
};

void 
DGrid::paintCell(QPainter *p,int row,int col)
{
	QColor temp;
	QPen temppen;
	//setAutoUpdate(FALSE);
	temp=p->backgroundColor();
	p->setBackgroundMode(OpaqueMode);
	if(index==TRUE && col==0)
	{
		p->setBackgroundColor(yellow);
	}
	else
	{
		p->setBackgroundColor(white);
	};
	p->eraseRect(1,1,cellWidth(col)-1,cellHeight()-1);
	p->drawLine(0,0,cellWidth(col)-1,0);
	p->drawLine(0,0,0,cellHeight()-1);
	if(row==numRows()-1)
	{
		p->drawLine(0,cellHeight()-1,cellWidth(col)-1,cellHeight()-1);
	};
	if(col==numCols()-1)
	{
		p->drawLine(cellWidth(col)-1,0,cellWidth(col)-1,cellHeight()-1);
	};
	if(col==curCol && row==curRow)
	{
		temppen=p->pen();
		p->setPen(blue);
		if(hasFocus()==FALSE)
		{
			p->setPen(DotLine);
		};
		p->drawRect(1,1,cellWidth(col)-1,cellHeight()-1);
		p->setPen(temppen);
	};
	p->setFont(*gridfont);
	p->drawText(5,0,cellWidth(col)-5,cellHeight(),AlignLeft | AlignVCenter,text[ICell(row,col)]);
	//p->setBackgroundMode(TransparentMode);
	p->setBackgroundColor(temp);
	//setAutoUpdate(TRUE);
	//repaint();
};

int 
DGrid::cellWidth(int col)
{
	if(col<numCols())
	{
		return width[col];
	}
	else
	{
		return 0;
	};
};

int 
DGrid::cellWidth()
{
	return 0;
};

void
DGrid::setWidth(int col,int val)
{
	width[col]=val;
	updateTableSize();
};

int 
DGrid::Cols(int cols=0)
{
	//int temp=numCols();
	if(cols==0)
	{
		return numCols();
	};
	if(text!=NULL)
	{
		delete[] text;
	};
	setNumCols(cols);
	updateTableSize();
	text=new QString[(numRows()*numCols())+2];
	return cols;
};

int 
DGrid::Rows(int rows=0)
{
	if(rows==0)
	{
		return numRows();
	};
	if(text!=NULL)
	{
		delete[] text;
	};
	setNumRows(rows);
	updateTableSize();
	text=new QString[(numRows()*numCols())+2];
	return rows;
};

void 
DGrid::mousePressEvent(QMouseEvent *event)
{
	int PrevCol;
	int PrevRow;
	QPoint point;
	if(event->button()==LeftButton)
	{
		point=event->pos();
		PrevCol=curCol;
		curCol=findCol(point.x());
		PrevRow=curRow;
		curRow=findRow(point.y());
		if((curCol!=0 && index==TRUE) || index==FALSE)
		{
			if(PrevCol!=curCol || PrevRow!=curRow)
			{
				updateCell(PrevRow,PrevCol);
				updateCell(curRow,curCol);
			};
			emit clicked();
		}
		else
		{
			curCol=PrevCol;
			curRow=PrevRow;
		};
	};
};

void 
DGrid::keyPressEvent(QKeyEvent *event)
{
	int temp;
	switch(event->key())
	{
		case Key_Left:
			if(curCol>0)
			{
				if(curCol!=1 || index!=TRUE)
				{
					if(leftCell()>curCol-1)
					{
						setLeftCell(curCol-1);
					};
					temp=curCol;
					curCol--;
					updateCell(curRow,temp);
					updateCell(curRow,curCol);
				}
				else
				{
					if(leftCell()>curCol-1)
					{
						setLeftCell(curCol-1);
					};
				};
			};
		break;
		case Key_Right:
			if(curCol<numCols()-1)
			{
					if(lastColVisible()<curCol+1)
					{
						setLeftCell(leftCell()+1);
					};
					temp=curCol;
					curCol++;
					updateCell(curRow,temp);
					updateCell(curRow,curCol);
			};
		break;
		case Key_Up:
			if(curRow>0)
			{
					if(topCell()>curRow-1)
					{
						setTopCell(topCell()-1);
					};
					temp=curRow;
					curRow--;
					updateCell(temp,curCol);
					updateCell(curRow,curCol);
			};
		break;
		case Key_Down:
			if(curRow<numRows()-1)
			{
					if(lastRowVisible()<curRow+1)
					{
						setTopCell(topCell()+1);
					};
					temp=curRow;
					curRow++;
					updateCell(temp,curCol);
					updateCell(curRow,curCol);
			};
		break;
		case Key_Enter:
			emit entered();
		break;
		default:
			event->ignore();
		break;
	};
};

int 
DGrid::CurCol(int col=-1)
{
	if(col==-1)
	{
		return curCol;
	};
	curCol=col;
	return col;
};

int 
DGrid::CurRow(int row=-1)
{
	if(row==-1)
	{
		return curRow;
	};
	curRow=row;
	return row;
};

bool 
DGrid::Index(int inx=-1)
{
	if(inx==-1)
	{
		return index;
	};
	if(inx==TRUE)
	{
		index=TRUE;
		if(curCol==0)
		{
			curCol=1;
		};
		return index;
	};
	index=FALSE;
	return index;
};

DGrid::~DGrid()
{
	if(text!=NULL)
	{
		delete[] text;
	};
	delete gridfont;
};

QString 
DGrid::Text(QString txt=NULL)
{
	if(txt==NULL)
	{
		return text[ICell(curRow,curCol)];
	};
	text[ICell(curRow,curCol)]=txt;
	return text[ICell(curRow,curCol)];
};

int  
DGrid::Text(int val=-1)
{
	if(val==-1)
	{
		return text[ICell(curRow,curCol)].toInt();
	};
	text[ICell(curRow,curCol)].setNum(val);
	return text[ICell(curRow,curCol)].toInt();
};

int 
DGrid::ICell(int row,int col)
{
	return (row*numCols())+col;
};
